% Code to generate the forecast error variance decompositions of the paper 
% by Laudati, Dario and M. Hashem Pesaran (2022). Identifying the Effects 
% of Sanctions on the Iranian Economy Using Newspaper Coverage, JAE, under 
% an alternative order specification with fx returns as first variable

close all; clear; clc;

tic
load('datamaster.mat')


%% Set-up the problem
% Load dataset from the "LP Sanctions Iran Data" folder

T = find(datamaster.date == "2020Q3");      % number of obs to be excluded (from 2020Q3 onwards)
E = find(datamaster.date == "1988Q4");      % number of obs to be excluded (39 obs up to 1988q4)

s = san_on./0.5 - 0.4*san_off./0.35;                % normalized sanctions variable
ls = lagmatrix(s,1);                                % s(t-1)

y = y_irn; yw = y_world; clearvars y_irn y_world;


% % Generate growth rates as log difference, and their lags
names   = {'fx_free' 'xoil' 'cpi' 'y' 'gep' 'geq'  'poil' 'yw'};
levels  = [fx_free,  xoil,   cpi,  y,  gep,  geq,  poil,  yw];

endog   = {'dfx' 'dxo' 'dp' 'dy'};
exog    = {'dyw'};
tot     = {'dfx' 'dxo' 'dp' 'dy' 'dyw' 's'};

for k = 1:length(names) 
  d.(names{1,k}) = log(levels(:,k)./lagmatrix(levels(:,k),1));
  ld.(names{1,k}) = lagmatrix(d.(names{1,k}),1) ;
end
l2dp = lagmatrix(ld.cpi,1);                         % We will need the second lag for prices
d.xoil = xoil./lagmatrix(xoil,1)-1;
ld.xoil = lagmatrix(d.xoil,1);

% Long-run interest rates are just in first difference
d.glr = glr./lagmatrix(glr,1)-1;     ld.glr = lagmatrix(d.glr,1);

% Matrices of rates of change
AUX1 = struct2cell(d);      AUX1 = AUX1';     D = cell2mat(AUX1);
AUX2 = struct2cell(ld);     AUX2 = AUX2';     LD = cell2mat(AUX2);
clear vars AUX*

all = [s, ls, D, grve, LD, l2dp, lagmatrix(grve,1)];
table_all = array2table(all, 'VariableNames', {'s', 's_lag', 'dfx', 'dxo', 'dp', 'dy', 'dgep', 'dgeq',...
    'dpoil', 'dyw', 'dglr', 'grv', 'dfxl1', 'dxol1', 'dpl1', 'dyl1', 'dgepl1', ...
    'dgeql1', 'dpoill1', 'dywl1', 'dglrl1', 'dpl2', 'grvel1'});


master = [datamaster, table_all];

n       = size(endog,2);                % Number of domestic variables
ncon    = size(exog,2) ;                % Number of controls (can be expanded by adding variables in the matrix)
ntot    = n + ncon +1;                  % Total number of equations: Domestic + controls + sanctions


%% Generate regressions estimates
% Foreign exchange returns equation
model_dfx = 'dfx ~ s + s_lag + dfxl1 + dxol1 + dpl1 + dyl1 + dyw ';
mdl_dfx = fitlm(master, model_dfx, 'Exclude', [1:E, T-2:T])

% Oil Exports equation
model_dxoil = 'dxo ~ s + s_lag + dfx + dfxl1 + dxol1 + dpl1 + dyl1 + dyw ';
mdl_dxo = fitlm(master, model_dxoil, 'Exclude', [1:E, T-2:T])

% Inflation equation
model_dp = 'dp ~ s + s_lag + dfx + dxo + dfxl1 + dxol1 + dpl1 + dyl1 + dpl2 + dyw ';
mdl_dp = fitlm(master, model_dp, 'Exclude', [1:E, T-2:T])

% Output growth equation
model_dy = 'dy ~ s + s_lag + dfx +  dxo + dp + dfxl1 + dxol1 + dpl1 + dyl1 + dyw';
mdl_dy = fitlm(master, model_dy, 'Exclude', [1:E, T-2:T])

% AR(1) model for sanctions intensity variable
model_s = 's ~ s_lag';
mdl_s = fitlm(master, model_s, 'Exclude', [1:E, T-2:T])
omega = mdl_s.RMSE; 
rho = table2array(mdl_s.Coefficients(2,1));

% Control variables model VAR(1) structure, if one wants to add more variables 
z = [d.yw];
z89 = z([E:T-3], :);
model_z = varm(ncon,1);
[est_z, est_SE, ~, est_res] = estimate(model_z, z89);
rho_dyw = cell2mat(est_z.AR);


mdl = {mdl_dfx, mdl_dxo, mdl_dp, mdl_dy};



% % Store residuals for each regression to be used later
Ttilde = T-3;
% Domestic variables
for k = 1:length(endog)
  epsilon.(endog{1,k}) = table2array(mdl{k}.Residuals(E+1:Ttilde,1));
end

%Exogenous variables
for k = 1:length(exog) 
  epsilon.(exog{1,k}) = est_res(:,k);
end

%Sanctions intensity variable
epsilon.s =  table2array(mdl_s.Residuals(E+1:Ttilde,1));

% Group shocks equal to 1 SE of each regression
for k=1:length(exog)
    RMSE.(exog{1,k}) = sqrt(sum(est_res(:,k).^2)/(length(est_res(:,k))-(ncon+1)));
end
RMSE.s = omega;

%% Parameter values to be inserted in the structural model

A0 = eye(n);   A1 = zeros(n,n);        A2 = zeros(n,n); 
C  = zeros(n,ncon);

% coefficients for contemporaneous effects
for ii = 2:length(endog)
    for jj = 1:ii-1
        A0(ii,jj) = - table2array(mdl{ii}.Coefficients(endog(jj),1));
    end
end

% coefficients for domestic vars. Lag = 1
lagvar = {'dfxl1', 'dxol1', 'dpl1', 'dyl1'};
for ii = 1:length(endog)
    for jj = 1:length(endog)
        A1(ii,jj) = table2array(mdl{ii}.Coefficients(lagvar(jj),1));
    end
end

% coefficients for domestic vars. lag=2
A2(length(endog)-1, length(endog)-1) = table2array(mdl_dp.Coefficients('dpl2',1));

aq = zeros(n,1);        azw = zeros(ncon,1);
gamma0 = zeros(n,1);    gamma1 = zeros(n,1);

% Intercepts to include
for ii = 1:length(endog)
    aq(ii,1) = table2array(mdl{ii}.Coefficients(1,1));
end

azw(:,1) = est_z.Constant(:,1);

as = table2array(mdl_s.Coefficients(1,1));

for ii = 1:length(endog)
    gamma0(ii,1) = table2array(mdl{ii}.Coefficients(2,1));
    gamma1(ii,1) = table2array(mdl{ii}.Coefficients(3,1));
    C(ii,1:ncon) = table2array(mdl{ii}.Coefficients('dyw',1));
    se(ii,1) = mdl{ii}.RMSE;
end

%%variance-covariance matrix
sigma = eye(n).*se.^2;

F1 = A0\A1;         F2 = A0\A2;


%%  Impulse Response Functions
% Loop to create the matrices G0, G1, ...
years = 5;          H = 4*years+1;       % % Number of quarters

G = cell(H,1);    G{1,1} = eye(n);        G{2,1} = F1; 

for i = 1:size(G,1)
    G{i+2,1} = F1*G{i+1,1} + F2*G{i,1};
end

J = inv(A0);

G_cumul = cell(size(G,1),1);
for h = 1:size(G)
    if h ==1
       G_cumul{h,1}(:,:) = G{h,1};
    else
       G_cumul{h,1}(:,:) = G{h,1} + G_cumul{h-1,1} ;
    end
end


% Standard single shock
for h=1:H
       a(h,1) = rho^(h-1);
       a_aux(1:n,1,h) = repelem(a(h,1),n)';
       if h==1
           dl(:,1,h) = G{h,1}*J*gamma0;   
           B_aux{h,1} = dl(:,1,1:h).*flip(a_aux(:,1,1:h),3);
       else
           dl(:,1,h) = G{h,1}*J*gamma0 + G{h-1,1}*J*gamma1;
           B_aux{h,1} = dl(:,1,1:h).*flip(a_aux(:,1,1:h),3);       
       end
 end
 
for h=1:H
     bl{h,1} = sum(B_aux{h,1},3);
 end






for h=1:H
	a(h,1) = rho^(h-1);
	a_aux(1:n,1,h) = repelem(a(h,1), n)';
        if h==1
           dlCUM(:,1,h) = G{h,1}*J*gamma0;   
           B_auxCUM{h,1} = dlCUM(:,1,1:h).*flip(a_aux(:,1,1:h),3);
        else
           dlCUM(:,1,h)= G_cumul{h,1}*J*gamma0 + G_cumul{h-1,1}*J*gamma1;
           B_auxCUM{h,1} = dlCUM(:,1,1:h).*flip(a_aux(:,1,1:h),3);
        end
end

for h=1:H
      blCUM{h,1} = sum(B_auxCUM{h,1},3);
end




for h=1:H
      a_dyw(h,1) = rho_dyw^(h-1);
      a_dyw_aux(1:n,1,h) = repelem(a_dyw(h,1),n)';
      cl(:,1,h) = G{h,1}*J*C;
      C_aux{h,1} = cl(:,1,1:h).*flip(a_dyw_aux(:,1,1:h),3);
end

for h=1:H
    kappa_l{h,1} = sum(C_aux{h,1},3);
end



eigen = eye(n);  e = cell(1,n);    %% generate canonical basis of order 3
for i = 1:n
    e{i} = eigen(i,:)';
end


%% FEVD
%denominator

den1 = cell(H,1);
den2 = cell(H,1);
den3 = cell(H,1);

for i=1:n
    for h=1:H
        den1{h,i} = e{i}'*G{h,1}*J*sigma*J'*G{h,1}'*e{i};
        den2{h,i} = e{i}'*blCUM{h,1}*blCUM{h,1}'*e{i};
        den3{h,i} = e{i}'*kappa_l{h,1}*kappa_l{h,1}'*e{i};
    end
end

den1_cumul = cell(h,i);
den2_cumul = cell(h,i);
den3_cumul = cell(h,i);

for i=1:n
    for h=1:H
        if h==1
            den1_cumul{h,i} = den1{h,i};
            den2_cumul{h,i} = den2{h,i};
            den3_cumul{h,i} = den3{h,i};
        else
            den1_cumul{h,i} = den1_cumul{h-1,i} + den1{h,i};
            den2_cumul{h,i} = den2_cumul{h-1,i} + den2{h,i};
            den3_cumul{h,i} = den3_cumul{h-1,i} + den3{h,i};
        end
    end
end

den = cell(H,n);
omega_dyw = RMSE.dyw ;

for i=1:n
    for h =1:H
        den2_cumul{h,i} = (omega^2)*den2_cumul{h,i};
        den3_cumul{h,i} = (omega_dyw^2)*den3_cumul{h,i};
        den{h,i} = den1_cumul{h,i} + den2_cumul{h,i} + den3_cumul{h,i};
    end
end

%numerator
num = zeros(n,n,H);

for i=1:n
    for j=1:n
        for h=1:H
            num(i,j,h)= (e{i}'*G{h,1}*J*e{j})^2;
        end
    end
end

num_cum = zeros(n,n,H);
for i=1:n
    for j=1:n
        for h=1:H
            if h==1
                num_cum(i,j,h)= num(i,j,h);
            else
                num_cum(i,j,h)= num(i,j,h) + num_cum(i,j,h-1);
            end
        end
    end
end

for i=1:n
    for j=1:n
        for h=1:H
            num_cum(i,j,h)= (sigma(j,j))*num_cum(i,j,h);
        end
    end
end


for i=1:n               %%add sanction shock
    for h=1:H
        num_cum(i,5,h)= den2_cumul{h,i};
        num_cum(i,6,h) = den3_cumul{h,i};        
    end
end


% Result
for h=1:H
    for i=1:n
        fevd{h,i} = num_cum(i,:,h)/den{h,i};
    end
end

for k = 1:length(endog) 
    FEVD.(endog{1,k}) = cell2mat(fevd(:,k));
end

%change the order of the variables for convenience
for k = 1:length(endog)
    FEVD.(endog{1,k}) = [FEVD.(endog{1,k})(:,n+1) FEVD.(endog{1,k})(:,1:n) FEVD.(endog{1,k})(:,n+2) ];
end


%% Plots
hh = 0:size(FEVD.dfx,1)-1;
low = zeros(size(FEVD.dfx,1),1);
set(0, 'DefaultAxesFontSize', 14, 'DefaultAxesFontName', 'Times');

for k = 1:length(endog)
graph.(endog{1,k})(:,1) = 100*FEVD.(endog{1,k})(:,1);
graph.(endog{1,k})(:,2) = 100*FEVD.(endog{1,k})(:,1) + 100*FEVD.(endog{1,k})(:,2);
graph.(endog{1,k})(:,3) = 100*FEVD.(endog{1,k})(:,1) + 100*FEVD.(endog{1,k})(:,2) + 100*FEVD.(endog{1,k})(:,3);
graph.(endog{1,k})(:,4) = 100*FEVD.(endog{1,k})(:,1) + 100*FEVD.(endog{1,k})(:,2) + 100*FEVD.(endog{1,k})(:,3)+ 100*FEVD.(endog{1,k})(:,4);
graph.(endog{1,k})(:,5) = 100*FEVD.(endog{1,k})(:,1) + 100*FEVD.(endog{1,k})(:,2) + 100*FEVD.(endog{1,k})(:,3)+ 100*FEVD.(endog{1,k})(:,4)+ 100*FEVD.(endog{1,k})(:,5);
graph.(endog{1,k})(:,6) = 100*FEVD.(endog{1,k})(:,1) + 100*FEVD.(endog{1,k})(:,2) + 100*FEVD.(endog{1,k})(:,3)+ 100*FEVD.(endog{1,k})(:,4)+ 100*FEVD.(endog{1,k})(:,5)+ 100*FEVD.(endog{1,k})(:,6);
end

my_blue = [0, 0.35, 0.8];
my_azul = [153, 204, 255]/255;

figure(1)   
set(gcf,'color','w','PaperUnits','normalized','WindowState','maximized');
plotFEVDcumsan = tiledlayout(1,4);
nexttile    
 shade(hh, low, 'w', ...
 hh, graph.dfx(:,1), 'w', ...
       hh, graph.dfx(:,2),'w',...
       hh, graph.dfx(:,3),'w', ...
       hh, graph.dfx(:,4), 'w', ...
       hh, graph.dfx(:,5), 'w', ...
       hh, graph.dfx(:,6), 'w', ...
       'FillType',[1 2; 2,1; 2 3; 3 2; 3 4; 4 3; 4 5; 5 4; 5 6; 6 5; 6 7; 7 6],...
       'FillColor',[ 0, 0.35, 0.8 ; 0, 0.35, 0.8;...
                    1, 0.6, 0.1 ; 1, 0.6, 0.1; ...
                    1, 0, 0; 1, 0, 0; ...
                    0.3 0.62 0 ; 0.3 0.62 0 ; ...
                    0.6  0.8  1.0; 0.6  0.8  1.0;...                                        
                    0.32  0.32  0.32; 0.32  0.32  0.32],...
       'FillAlpha', 0.7);
   hold on 
    xlabel('\textit{Quarter ahead}',  'interpreter', 'latex', 'FontSize',20)
    title('Exchange rate','FontSize',20, 'interpreter', 'latex');    
    subtitle({'IRR/USD'},'FontSize',14, 'interpreter', 'latex')    
    xlim([0, 20])
    xticks(0:4:20)
    ylim([0, 100])
    hAx = gca;
    ylabel('Per cent', 'FontSize',13, 'interpreter', 'latex')
    ylh = get(gca,'ylabel');
    gyl = get(ylh);                                                         % Object Information
    ylp = get(ylh, 'Position');
    set(ylh, 'Rotation',360, 'Position', [-1.614,105.5,-1])
    axis square
nexttile    
    shade(hh, low, 'w', ...
       hh, graph.dxo(:,1), 'w', ...
       hh, graph.dxo(:,2),'w',...
       hh, graph.dxo(:,3),'w', ...
       hh, graph.dxo(:,4), 'w', ...
       hh, graph.dxo(:,5), 'w', ...
       hh, graph.dxo(:,6), 'w', ...
       'FillType',[1 2; 2,1; 2 3; 3 2; 3 4; 4 3; 4 5; 5 4; 5 6; 6 5; 6 7; 7 6],...
       'FillColor',[ 0, 0.35, 0.8 ; 0, 0.35, 0.8;...
                    1, 0.6, 0.1 ; 1, 0.6, 0.1; ...
                    1, 0, 0; 1, 0, 0; ...
                    0.3 0.62 0 ; 0.3 0.62 0 ; ...
                    0.6  0.8  1.0; 0.6  0.8  1.0;...                                        
                    0.32  0.32  0.32; 0.32  0.32  0.32],...
       'FillAlpha', 0.7);
   hold on 
    xlabel('\textit{Quarter ahead}',  'interpreter', 'latex', 'FontSize',20)
    title('Oil exports','FontSize',20, 'interpreter', 'latex');    
    subtitle({' '},'FontSize',14, 'interpreter', 'latex')    
    xlim([0, 20])
    xticks(0:4:20)
    ylim([0, 100])
    hAx = gca;
    ylabel('Per cent', 'FontSize',13, 'interpreter', 'latex')
    ylh = get(gca,'ylabel');
    gyl = get(ylh);                                                         % Object Information
    ylp = get(ylh, 'Position');
    set(ylh, 'Rotation',360, 'Position', [-1.614,105.5,-1])
    axis square
nexttile
 shade(hh, low, 'w', ...
 hh, graph.dp(:,1), 'w', ...
       hh, graph.dp(:,2),'w',...
       hh, graph.dp(:,3),'w', ...
       hh, graph.dp(:,4), 'w', ...
       hh, graph.dp(:,5), 'w', ...
       hh, graph.dp(:,6), 'w', ...
       'FillType',[1 2; 2,1; 2 3; 3 2; 3 4; 4 3; 4 5; 5 4; 5 6; 6 5; 6 7; 7 6],...
       'FillColor',[ 0, 0.35, 0.8 ; 0, 0.35, 0.8;...
                    1, 0.6, 0.1 ; 1, 0.6, 0.1; ...
                    1, 0, 0; 1, 0, 0; ...
                    0.3 0.62 0 ; 0.3 0.62 0 ; ...
                    0.6  0.8  1.0; 0.6  0.8  1.0;...                                        
                    0.32  0.32  0.32; 0.32  0.32  0.32],...
                           'FillAlpha', 0.7);
   hold on 
    xlabel('\textit{Quarter ahead}',  'interpreter', 'latex', 'FontSize',20)
    title('Inflation','FontSize',20, 'interpreter', 'latex');    
    subtitle(' ','FontSize',13, 'interpreter', 'latex');        
    xlim([0, 20])
    xticks(0:4:20)
    ylim([0, 100])
%     ytickformat('percentage')
    hAx = gca;
%     set(hAx,'YTickLabel',{'0','','20','','40','','60','','80','','100'})
    ylabel('Per cent', 'FontSize',13, 'interpreter', 'latex')
    ylh = get(gca,'ylabel');
    gyl = get(ylh);                                                         % Object Information
    ylp = get(ylh, 'Position');
    set(ylh, 'Rotation',360, 'Position', [-1.614,105.5,-1])
    axis square    
    leg =   legend({'','','','','','','','Cumulated sanctions intensity', 'Exchange rate',...
              'Oil exports','Inflation', 'Iran output growth','', '','World output growth'},...
              'Location','southoutside','NumColumns',3, 'Fontsize', 16,...
               'interpreter', 'latex');   
    new_width = leg.Position(4)*1.1;
    leg.Position(2) = leg.Position(2) - (new_width - leg.Position(4));
    new_width1 = leg.Position(1)*10.3;
    leg.Position(1) = leg.Position(1) - (new_width1 - leg.Position(1));
nexttile    
 shade(hh, low, 'w', ...
 hh, graph.dy(:,1), 'w', ...
       hh, graph.dy(:,2),'w',...
       hh, graph.dy(:,3),'w', ...
       hh, graph.dy(:,4), 'w', ...
       hh, graph.dy(:,5), 'w', ...
       hh, graph.dy(:,6), 'w', ...
       'FillType',[1 2; 2,1; 2 3; 3 2; 3 4; 4 3; 4 5; 5 4; 5 6; 6 5; 6 7; 7 6],...
       'FillColor',[ 0, 0.35, 0.8 ; 0, 0.35, 0.8;...
                    1, 0.6, 0.1 ; 1, 0.6, 0.1; ...
                    1, 0, 0; 1, 0, 0; ...
                    0.3 0.62 0 ; 0.3 0.62 0 ; ...
                    0.6  0.8  1.0; 0.6  0.8  1.0;...                                        
                    0.32  0.32  0.32; 0.32  0.32  0.32],...
       'FillAlpha', 0.7);
   hold on 
    xlabel('\textit{Quarter ahead}',  'interpreter', 'latex', 'FontSize',20)
    title('Iran output growth','FontSize',20, 'interpreter', 'latex');
    subtitle(' ','FontSize',13, 'interpreter', 'latex');        
    xlim([0, 20])
    xticks(0:4:20)
    ylim([0, 100])
    hAx = gca;
%     set(hAx,'YTickLabel',{'0','','20','','40','','60','','80','','100'})
    ylabel('Per cent', 'FontSize',13, 'interpreter', 'latex')
    ylh = get(gca,'ylabel');
    gyl = get(ylh);                                                         % Object Information
    ylp = get(ylh, 'Position');
    set(ylh, 'Rotation',360, 'Position', [-1.614,105.5,-1])
    axis square        
exportgraphics(plotFEVDcumsan,'../LP Iran Sanctions Results/SVAR estimates/IRFs_FEVDs_results/FEVD_cumSshock_DFXorder.png','Resolution',450)
