% Code to generate the forecast error variance decompositions of the paper 
% by Laudati, Dario and M. Hashem Pesaran (2022). Identifying the Effects 
% of Sanctions on the Iranian Economy Using Newspaper Coverage. JAE


close all; clear; clc;


%% Set-up the problem
% Load dataset from the "LP Sanctions Iran Data" folder
load('datamaster.mat')

tic 

T = find(datamaster.date=="2020Q3");      % number of obs to be excluded (from 2020Q3 onwards)
E = find(datamaster.date=="1988Q4");      % number of obs to be excluded (39 obs up to 1988q4)

s = san_on./0.5 - 0.4*san_off./0.35;                % normalized sanctions variable
ls = lagmatrix(s,1);                                % s(t-1)

y = y_irn; yw = y_world; clearvars y_irn y_world;


% % Generate growth rates as log difference, and their lags
names   = {'xoil' 'fx_free' 'cpi' 'y' 'gep' 'geq'  'poil' 'yw'};
levels  = [xoil,   fx_free,  cpi,  y,  gep,  geq,  poil,  yw];

for k = 1:length(names) 
  d.(names{1,k}) = log(levels(:,k)./lagmatrix(levels(:,k),1));
  ld.(names{1,k}) = lagmatrix(d.(names{1,k}),1) ;
end
l2dp = lagmatrix(ld.cpi,1); %% we will need the second lag for prices

d.xoil = xoil./lagmatrix(xoil,1)-1;
ld.xoil = lagmatrix(d.xoil,1);

%%Long-run interest rates are just in first difference
d.glr = glr./lagmatrix(glr,1)-1;     ld.glr = lagmatrix(d.glr,1);

%%Matrices of rates of change
AUX1 = struct2cell(d);      AUX1 = AUX1';     D = cell2mat(AUX1);
AUX2 = struct2cell(ld);     AUX2 = AUX2';     LD = cell2mat(AUX2);
clear vars AUX*

all = [s, ls, D, grve, LD, l2dp, lagmatrix(grve,1)];
table_all = array2table(all, 'VariableNames', {'s', 's_lag', 'dxo', 'dfx', 'dp', 'dy', 'dgep', 'dgeq',...
    'dpoil', 'dyw', 'dglr', 'grv', 'dxol1', 'dfxl1', 'dpl1', 'dyl1', 'dgepl1', ...
    'dgeql1', 'dpoill1', 'dywl1', 'dglrl1', 'dpl2', 'grvel1'});


master = [datamaster, table_all];


endog   = {'dxo' 'dfx' 'dp' 'dy'};
exog    = {'dyw'};
tot     = {'dxo' 'dfx' 'dp' 'dy' 'dyw' 's'};

n       = length(endog);           % Number of domestic variables
ncon    = length(exog) ;           % Number of controls (can be expanded by adding variables in the matrix)
ntot    = n + ncon +1;             % Total number of equations: Domestic + controls + sanctions

%% Generate regressions estimates

% Oil Exports equation
model_dxoil = 'dxo ~ s + s_lag + dxol1 + dfxl1 + dpl1 + dyl1 + dyw ';
mdl_dxo = fitlm(master, model_dxoil, 'Exclude', [1:E, T-2:T])

% Foreign exchange returns equation
model_dfx = 'dfx ~ s + s_lag + dxo + dxol1 + dfxl1 + dpl1 + dyl1 + dyw ';
mdl_dfx = fitlm(master, model_dfx, 'Exclude', [1:E, T-2:T])

% Inflation equation
model_dp = 'dp ~ s + s_lag + dxo + dfx + dxol1 + dfxl1 + dpl1 + dyl1 + dpl2 + dyw ';
mdl_dp = fitlm(master, model_dp, 'Exclude', [1:E, T-2:T])

% Output growth equation
model_dy = 'dy ~ s + s_lag +  dxo + dfx + dp + dxol1 + dfxl1 + dpl1 + dyl1 + dyw';
mdl_dy = fitlm(master, model_dy, 'Exclude', [1:E, T-2:T])

% AR(1) model for sanctions intensity variable
model_s = 's ~ s_lag';
mdl_s = fitlm(master, model_s, 'Exclude', [1:E, T-2:T])
omega = mdl_s.RMSE; 
rho = table2array(mdl_s.Coefficients(2,1));


% Build cell structure on models to be used later
mdl = {mdl_dxo, mdl_dfx, mdl_dp, mdl_dy};

% Control variables model VAR(1) structure, if one wants to add more variables 
Ttilde  = T-3;
z       = [d.yw];
z89     = z([E:Ttilde], :);
model_z = varm(ncon,1);
[est_z, est_SE, ~, est_res] = estimate(model_z, z89);



% % Store residuals for each regression to be used later
% Domestic variables
for k = 1:length(endog)
    epsilon.(endog{1,k}) = table2array(mdl{k}.Residuals(E+1:Ttilde,1));
end

% Exogenous variables
for k = 1:length(exog)
    epsilon.(exog{1,k}) = est_res(:,k);
end

% Sanctions intensity variable
epsilon.s =  table2array(mdl_s.Residuals(E+1:Ttilde,1));

% Group shocks equal to 1 SE of each regression
for k=1:length(exog)
    RMSE.(exog{1,k}) = sqrt(sum(est_res(:,k).^2)/(length(est_res(:,k))-(ncon+1)));
end
RMSE.s = omega;

%% Parameter values to be inserted in the structural model

A0 = eye(n);   A1 = zeros(n,n);        A2 = zeros(n,n);
Dw = zeros(n,ncon);

% coefficients for contemporaneous effects
for ii = 2:length(endog)
    for jj = 1:ii-1
        A0(ii,jj) = - table2array(mdl{ii}.Coefficients(endog(jj),1));
    end
end


% coefficients for domestic vars. Lag = 1
lagvar = {'dxol1', 'dfxl1', 'dpl1', 'dyl1'};
for ii = 1:length(endog)
    for jj = 1:length(endog)
        A1(ii,jj) = table2array(mdl{ii}.Coefficients(lagvar(jj),1));
    end
end

% coefficients for domestic vars. lag=2
A2(length(endog)-1, length(endog)-1) = table2array(mdl_dp.Coefficients('dpl2',1));

aq = zeros(n,1);        azw = zeros(ncon,1);
gamma0 = zeros(n,1);    gamma1 = zeros(n,1);

% Intercepts to include
for ii = 1:length(endog)
    aq(ii,1) = table2array(mdl{ii}.Coefficients(1,1));
end

azw(:,1)    = est_z.Constant(:,1);
as          = table2array(mdl_s.Coefficients(1,1));

for ii = 1:length(endog)
    gamma0(ii,1)    = table2array(mdl{ii}.Coefficients(2,1));
    gamma1(ii,1)    = table2array(mdl{ii}.Coefficients(3,1));
    Dw(ii,1:ncon)   = table2array(mdl{ii}.Coefficients('dyw',1));
    se(ii,1)        = mdl{ii}.RMSE;
end

% standard errors of the regressions
se = [se; RMSE.dyw; RMSE.s ];

 
% % variance-covariance matrix
sigma = eye(ntot).*se.^2;


% Build a single system of matrices
P0 = [A0           ,  -Dw               ,  -gamma0;
     zeros(ncon, n),  eye(ncon)         ,  zeros(ncon,1);
     zeros(1,n)    ,  zeros(1,ncon)     ,     1        ];

P1 = [A1            ,  zeros(n, ncon)   , gamma1;
    zeros(ncon, n)  ,  est_z.AR{1,1}    , zeros(ncon, 1);
    zeros(1,n)      ,  zeros(1, ncon)   , rho];

P2 = [A2, zeros(n, ncon), zeros(n,1);
        zeros(ncon, ntot);
        zeros(1, ntot)];

a = [aq ; azw; as];

% Reduced form matrices
F1 = P0\P1;         F2 = P0\P2;

% Loop to create the matrices G0, G1, ...
years = 5;          H = 4*years;       % Number of quarters

G = cell(H,1);    G{1,1} = eye(ntot);        G{2,1} = F1; 

for i = 1:size(G,1)
    G{i+2,1} = F1*G{i+1,1} + F2*G{i,1};
end


%% Variables shocks

% Domestic variables
for ind = 1:length(tot)
    shock.(tot{1,ind}) =  se(ind,1);
    IR.(tot{1,ind}) = zeros(ntot,H);
end

% Generate canonical bases of order ntot
eigen = eye(ntot);  e = cell(1, ntot);
for i = 1:ntot
    e{i} = eigen(i,:)';
end

% All impulse responses for all shocks
for ind = 1:length(tot)
    for h = 1:H
        IR.(tot{1,ind})(:,h) = (G{h,1}*(inv(P0))*e{ind})*shock.(tot{1,ind});
    end
end


%% FEVDs
% % variance-covariance matrix
sigma = eye(ntot).*se.^2;

% Denominators
den = cell(H,ntot);

for i = 1:ntot
    for h = 1:H
        den{h,i} = e{i}'*G{h,1}*(inv(P0))*sigma*(inv(P0'))*(G{h,1})'*e{i};
    end
end

den_cumul = cell(h,i);
for i = 1:ntot
    for h = 1:H
        if h == 1
            den_cumul{h,i} = den{h,i};
        else
            den_cumul{h,i} = den_cumul{h-1,i} + den{h,i};
        end
    end
end


%% Numerators
num = zeros(ntot, ntot, H);

%columns are the variables shocked, rows are the variables reacting
for i = 1:ntot
    for j = 1:ntot
        for h = 1:H
            num(i,j,h)= (e{i}'*G{h,1}*(inv(P0))*e{j})^2;
        end
    end
end


num_cum = zeros(ntot, ntot, H);
for i = 1:ntot
    for j = 1:ntot
        for h = 1:H
            if h == 1
                num_cum(i,j,h)= num(i,j,h);
            else
                num_cum(i,j,h)= num(i,j,h) + num_cum(i,j,h-1);
            end
        end
    end
end

for i = 1:ntot
    for j = 1:ntot
        for h = 1:H
            num_cum(i,j,h)= (sigma(j,j))*num_cum(i,j,h);
        end
    end
end

% Result
for h = 1:H
    for i = 1:ntot
        fevd{h,i} = num_cum(i,:,h)/den_cumul{h,i};
    end
end

% Regroup by Variance contribution of variables shocks for each variable
for k = 1:length(tot) 
	FEVD.(tot{1,k}) = cell2mat(fevd(:,k));
end


%change the order of the variables for convenience in the final output
for k = 1:length(tot) 
    FEVD.(tot{1,k}) = [FEVD.(tot{1,k})(:,end) FEVD.(tot{1,k})(:,1:end-1)];
end

quarter = 0:H-1; quarter = quarter';
dxo = array2table([quarter FEVD.dxo],   'VariableNames', {'Quarter ahead' 's' 'dxo' 'dfx' 'dp' 'dy' 'dyw'});
dfx = array2table([quarter FEVD.dfx],   'VariableNames', {'Quarter ahead' 's' 'dxo' 'dfx' 'dp' 'dy' 'dyw'});
dp  = array2table([quarter FEVD.dp],    'VariableNames', {'Quarter ahead' 's' 'dxo' 'dfx' 'dp' 'dy' 'dyw'});
dy  = array2table([quarter FEVD.dy],    'VariableNames', {'Quarter ahead' 's' 'dxo' 'dfx' 'dp' 'dy' 'dyw'});

filename = 'fevd/FEVDs.xlsx';
writetable(dxo, '../LP Iran Sanctions Results/SVAR estimates/IRFs_FEVDs_results/FEVDs_main.xlsx', 'Sheet', 'FEVD for dxo', 'Range', 'A1:G10');
writetable(dfx, '../LP Iran Sanctions Results/SVAR estimates/IRFs_FEVDs_results/FEVDs_main.xlsx', 'Sheet', 'FEVD for dfx', 'Range', 'A1:G10');
writetable(dp,  '../LP Iran Sanctions Results/SVAR estimates/IRFs_FEVDs_results/FEVDs_main.xlsx', 'Sheet', 'FEVD for dp', 'Range', 'A1:G10');
writetable(dy,  '../LP Iran Sanctions Results/SVAR estimates/IRFs_FEVDs_results/FEVDs_main.xlsx', 'Sheet', 'FEVD for dy', 'Range', 'A1:G10');


toc