% Code to generate the impulse response function results of the paper by
% Laudati, Dario and M. Hashem Pesaran (2022). Identifying the Effects of
% Sanctions on the Iranian Economy Using Newspaper Coverage. JAE, under an
% alternative order specification with fx returns as first variable

close all; clear; clc;

tic
%% Set-up the problem
% Load dataset from the "LP Sanctions Iran Data" folder
load('datamaster.mat')

T = find(datamaster.date=="2020Q3");      % number of obs to be excluded (from 2020Q3 onwards)
E = find(datamaster.date=="1988Q4");      % number of obs to be excluded (39 obs up to 1988q4)

s   = san_on./0.5 - 0.4*san_off./0.35;                % normalized sanctions variable
ls  = lagmatrix(s,1);                                % s(t-1)

y = y_irn; yw = y_world; clearvars y_irn y_world;


% % Generate growth rates as log difference, and their lags
names   = {'fx_free' 'xoil' 'cpi' 'y' 'gep' 'geq'  'poil' 'yw'};
levels  = [fx_free,  xoil,   cpi,  y,  gep,  geq,  poil,  yw];

for k = 1:length(names) 
  d.(names{1,k}) = log(levels(:,k)./lagmatrix(levels(:,k),1));
  ld.(names{1,k}) = lagmatrix(d.(names{1,k}),1) ;
end
l2dp = lagmatrix(ld.cpi,1); %% we will need the second lag for prices

d.xoil = xoil./lagmatrix(xoil,1)-1;
ld.xoil = lagmatrix(d.xoil,1);

%%Long-run interest rates are just in first difference
d.glr = glr./lagmatrix(glr,1)-1;     ld.glr = lagmatrix(d.glr,1);

%%Matrices of rates of change
AUX1 = struct2cell(d);      AUX1 = AUX1';     D = cell2mat(AUX1);
AUX2 = struct2cell(ld);     AUX2 = AUX2';     LD = cell2mat(AUX2);
clear vars AUX*

all = [s, ls, D, grve, LD, l2dp, lagmatrix(grve,1)];
table_all = array2table(all, 'VariableNames', {'s', 's_lag', 'dfx', 'dxo', 'dp', 'dy', 'dgep', 'dgeq',...
    'dpoil', 'dyw', 'dglr', 'grv', 'dfxl1', 'dxol1', 'dpl1', 'dyl1', 'dgepl1', ...
    'dgeql1', 'dpoill1', 'dywl1', 'dglrl1', 'dpl2', 'grvel1'});


master = [datamaster, table_all];


endog   = {'dfx' 'dxo' 'dp' 'dy'};
exog    = {'dyw'};
tot     = {'dfx' 'dxo' 'dp' 'dy' 'dyw' 's'};

n       = length(endog);           % Number of domestic variables
ncon    = length(exog) ;           % Number of controls (can be expanded by adding variables in the matrix)
ntot    = n + ncon +1;             % Total number of equations: Domestic + controls + sanctions

%% Generate regressions estimates
% Foreign exchange returns equation
model_dfx = 'dfx ~ s + s_lag + dfxl1 + dxol1 + dpl1 + dyl1 + dyw ';
mdl_dfx = fitlm(master, model_dfx, 'Exclude', [1:E, T-2:T])

% Oil Exports equation
model_dxoil = 'dxo ~ s + s_lag + dfx + dfxl1 + dxol1 + dpl1 + dyl1 + dyw ';
mdl_dxo = fitlm(master, model_dxoil, 'Exclude', [1:E, T-2:T])

% Inflation equation
model_dp = 'dp ~ s + s_lag + dfx + dxo + dfxl1 + dxol1 + dpl1 + dyl1 + dpl2 + dyw ';
mdl_dp = fitlm(master, model_dp, 'Exclude', [1:E, T-2:T])

% Output growth equation
model_dy = 'dy ~ s + s_lag + dfx +  dxo + dp + dfxl1 + dxol1 + dpl1 + dyl1 + dyw';
mdl_dy = fitlm(master, model_dy, 'Exclude', [1:E, T-2:T])

% AR(1) model for sanctions intensity variable
model_s = 's ~ s_lag';
mdl_s = fitlm(master, model_s, 'Exclude', [1:E, T-2:T])
omega = mdl_s.RMSE; 
rho = table2array(mdl_s.Coefficients(2,1));


% Build cell structure on models to be used later
mdl = {mdl_dfx, mdl_dxo, mdl_dp, mdl_dy};

% Control variables model VAR(1) structure, if one wants to add more variables 
Ttilde  = T-3;
z       = [d.yw];
z89     = z([E:Ttilde], :);
model_z = varm(ncon,1);
[est_z, est_SE, ~, est_res] = estimate(model_z, z89);



% % Store residuals for each regression to be used later
% Domestic variables
for k = 1:length(endog)
    epsilon.(endog{1,k}) = table2array(mdl{k}.Residuals(E+1:Ttilde,1));
end

% Exogenous variables
for k = 1:length(exog)
    epsilon.(exog{1,k}) = est_res(:,k);
end

% Sanctions intensity variable
epsilon.s =  table2array(mdl_s.Residuals(E+1:Ttilde,1));

% Group shocks equal to 1 SE of each regression
for k=1:length(exog)
    RMSE.(exog{1,k}) = sqrt(sum(est_res(:,k).^2)/(length(est_res(:,k))-(ncon+1)));
end
RMSE.s = omega;

%% Parameter values to be inserted in the structural model

A0 = eye(n);   A1 = zeros(n,n);        A2 = zeros(n,n);
Dw = zeros(n,ncon);

% coefficients for contemporaneous effects
for ii = 2:length(endog)
    for jj = 1:ii-1
        A0(ii,jj) = - table2array(mdl{ii}.Coefficients(endog(jj),1));
    end
end


% coefficients for domestic vars. Lag = 1
lagvar = {'dfxl1', 'dxol1', 'dpl1', 'dyl1'};
for ii = 1:length(endog)
    for jj = 1:length(endog)
        A1(ii,jj) = table2array(mdl{ii}.Coefficients(lagvar(jj),1));
    end
end

% coefficients for domestic vars. lag=2
A2(length(endog)-1, length(endog)-1) = table2array(mdl_dp.Coefficients('dpl2',1));

aq = zeros(n,1);        azw = zeros(ncon,1);
gamma0 = zeros(n,1);    gamma1 = zeros(n,1);

% Intercepts to include
for ii = 1:length(endog)
    aq(ii,1) = table2array(mdl{ii}.Coefficients(1,1));
end

azw(:,1)    = est_z.Constant(:,1);
as          = table2array(mdl_s.Coefficients(1,1));

for ii = 1:length(endog)
    gamma0(ii,1)    = table2array(mdl{ii}.Coefficients(2,1));
    gamma1(ii,1)    = table2array(mdl{ii}.Coefficients(3,1));
    Dw(ii,1:ncon)   = table2array(mdl{ii}.Coefficients('dyw',1));
    se(ii,1)        = mdl{ii}.RMSE;
end

% standard errors of the regressions
se = [se; RMSE.dyw; RMSE.s ];

 
% % variance-covariance matrix
sigma = eye(ntot).*se.^2;


% Build a single system of matrices
P0 = [A0           ,  -Dw               ,  -gamma0;
     zeros(ncon, n),  eye(ncon)         ,  zeros(ncon,1);
     zeros(1,n)    ,  zeros(1,ncon)     ,     1        ];

P1 = [A1            ,  zeros(n, ncon)   , gamma1;
    zeros(ncon, n)  ,  est_z.AR{1,1}    , zeros(ncon, 1);
    zeros(1,n)      ,  zeros(1, ncon)   , rho];

P2 = [A2, zeros(n, ncon), zeros(n,1);
        zeros(ncon, ntot);
        zeros(1, ntot)];

a = [aq ; azw; as];

% Reduced form matrices
F1 = P0\P1;         F2 = P0\P2;

% Loop to create the matrices G0, G1, ...
years = 5;          H = 4*years;       % Number of quarters

G = cell(H,1);    G{1,1} = eye(ntot);        G{2,1} = F1; 

for i = 1:size(G,1)
    G{i+2,1} = F1*G{i+1,1} + F2*G{i,1};
end


%% Variables shocks

% Domestic variables
for ind = 1:length(tot)
    shock.(tot{1,ind}) =  se(ind,1);
    IR.(tot{1,ind}) = zeros(ntot,H);
end

% Generate canonical bases of order ntot
eigen = eye(ntot);  e = cell(1, ntot);
for i = 1:ntot
    e{i} = eigen(i,:)';
end

% All impulse responses for all shocks
for ind = 1:length(tot)
    for h = 1:H
        IR.(tot{1,ind})(:,h) = (G{h,1}*(inv(P0))*e{ind})*shock.(tot{1,ind});
    end
end


%% ################################################################### %%
    % ####################### BOOTSTRAPPING ####################### %
 % ################################################################### %

% Need to draw random samples by shuffling the estimated residuals (w/ replacement)

R = 1000;       %% Number of replications
rng(81);        %% specify seed

% Resample from the third observation on (first 2 taken as given to initialize the process later)
samplesize = size(3:length(epsilon.dfx),2);
for ind = 1:length(tot)
    for r = 1:R
        resBP.(tot{1,ind})(1:samplesize,r) = datasample(epsilon.(tot{1,ind})(3:length(epsilon.(tot{1,ind})),1), samplesize);
    end
end


% Group all (m+k+1) residuals series by replication, i.e. 1 replication per cell.
rBP = cell(1,R);
for r = 1:R
    rBP{1,r} = [resBP.dfx(:,r), resBP.dxo(:,r), resBP.dp(:,r), resBP.dy(:,r), resBP.dyw(:,r), resBP.s(:,r)];
end


%% Generate simulated variables

% stack (m+k+1) variables in z-vectors to initialize the process
q = [d.fx_free(E+1:Ttilde), d.xoil(E+1:Ttilde),  d.cpi(E+1:Ttilde), d.y(E+1:Ttilde), d.yw(E+1:Ttilde), s(E+1:Ttilde)];

% Make each column to be a quarter, i.e. a ((m+k+1) x 1) vector.
q = q';
numblocks = T-E-3;
Q = mat2cell(q, ntot, repelem(1,  numblocks));  % each cell is a quarter for all variables

% Transform the residuals to have a ((m+k+1) x 1) vector for each period and for each replication 
U = cell(numblocks-2, R);
for r = 1:R
    for i = 1:samplesize
        U{i,r}(:,1) = rBP{1,r}(i,:) ; 
    end
end

% Begin simulating z(r). The first two observations are given from original data
Qr = cell(R,numblocks);
for r = 1:R
    for j = 1:2
        Qr{r,j} = Q{1,j};
    end
end

%recursive formulation for the obs from 1989q3
for r = 1:R
    for i = 3:numblocks
        Qr{r,i} = P0\a + F1*Qr{r,i-1} + F2*Qr{r,i-2}  + P0\U{i-2,r};
    end
end

%% Simulate estimates of endogenous variables coefficients 

% Regroup variable-wise over the whole time horizon
for ind = 1:length(tot)
    BP.(tot{1,ind}) = zeros(numblocks,R);
end
    
for ind = 1:length(tot)
    for r = 1:R
        for i = 1:numblocks
            BP.(tot{1,ind})(i,r) = Qr{r,i}(ind);
        end
    end
end

names = {'fx_free', 'xoil', 'cpi','y', 'yw', 's'};

for ind = 1:length(names)-1
   aux.(names{1,ind}) = d.(names{1,ind});
end
aux.s = s;

% Lagged variables to run regressions (except sanctions, which we don't simulate)
for ind = 1:length(tot)
    BPl1.(tot{1,ind})           = lagmatrix(BP.(tot{1,ind}),1);
    BPl1.(tot{1,ind})(1,:)      = aux.(names{1,ind})(E);                         % we know the first obs
    BPl2.(tot{1,ind})           = lagmatrix(BP.(tot{1,ind}),2);
    BPl2.(tot{1,ind})([1 2],:)  = aux.(names{1,ind})(repmat([E-1; E],1,R));  % we know the first 2 obs
end

inpt = ones(numblocks,1);

% store regression results

for r = 1:R
    [b.dfx(:,r), ~, ~, ~, stats.dfx(:,r)] =  regress(BP.dfx(:,r), [inpt, BP.s(:,r), BPl1.s(:,r), ...
        BPl1.dfx(:,r), BPl1.dxo(:,r), BPl1.dp(:,r), BPl1.dy(:,r), BP.dyw(:,r) ]);
       
    [b.dxo(:,r), ~, ~, ~, stats.dxo(:,r)] =  regress(BP.dxo(:,r), [inpt, BP.s(:,r), BPl1.s(:,r), ...
        BP.dfx(:,r), BPl1.dfx(:,r), BPl1.dxo(:,r), BPl1.dp(:,r), BPl1.dy(:,r), BP.dyw(:,r)]);
    
    [b.dp(:,r), ~, ~, ~, stats.dp(:,r)] =  regress(BP.dp(:,r), [inpt, BP.s(:,r), BPl1.s(:,r), ...
        BP.dfx(:,r), BP.dxo(:,r), BPl1.dfx(:,r), BPl1.dxo(:,r), BPl1.dp(:,r), BPl1.dy(:,r), ...
        BPl2.dp(:,r), BP.dyw(:,r)]);

    [b.dy(:,r), ~,~,~,stats.dy(:,r)] = regress(BP.dy(:,r), [inpt,  BP.s(:,r), BPl1.s(:,r), ...
        BP.dfx(:,r), BP.dxo(:,r), BP.dp(:,r), BPl1.dfx(:,r),  BPl1.dxo(:,r), BPl1.dp(:,r), ...
        BPl1.dy(:,r), BP.dyw(:,r)]);        
end

% Structure originally intended for a larger set of exogenous control
% variables, can be expanded in this respect
for ind = 1:length(exog)
    for r = 1:R
        [b.(exog{1,ind})(:,r), ~,~,~, stats.(exog{1,ind})(:,r)] =  regress(BP.(exog{1,ind})(:,r), ...
            [inpt, BPl1.dyw(:,r) ]);
    end
end

for r = 1:R
    [b.s(:,r), ~, ~, ~, stats.s(:,r)] = regress(BP.s(:,r), [inpt, BPl1.s(:,r)]);
end

% standard error of the regression for each replication. Except sanctions
% which I do not simulate
for ind = 1:length(tot)
    rmse.(tot{1,ind}) = sqrt(stats.(tot{1,ind})(4,:))';
end

% Write ("structural") matrices to perform IRFs
I = repmat(eye(n),1,R);
zero = repmat(zeros(n,n),1,R);
zerocon = repmat(zeros(n,ncon),1,R);

A0R = mat2cell(I, n, repelem(n,R));
A1R = mat2cell(zero, n, repelem(n,R));
A2R = mat2cell(zero, n, repelem(n,R));
DwR = mat2cell(zerocon, n, repelem(ncon,R));

gamma0BP    = zeros(n,R);          gamma1BP = zeros(n,R);
AzwR        = cell(1,R);

F1R = cell(1,R);    F2R = cell(1,R);
G_R = cell(H,R);

% Generate the Structural Matrices for SVAR for each replication
for r = 1:R
    for ii =2 :length(endog)
        for jj = 1:ii-1
            A0R{1,r}(ii,jj) = - b.(char(endog(ii)))(3+jj, r);
        end
    end
end

for r = 1:R
    for ii = 1:length(endog)
        for jj = 1:length(endog)
            A1R{1,r}(ii,jj) = b.(char(endog(ii)))(3+jj+ii-1, r);
        end
    end
end

for r = 1:R
    A2R{1,r}(length(endog)-1, length(endog)-1) = b.dp(10,r);
end

for r = 1:R
    for ii = 1:length(endog)
        DwR{1,r}(ii,1:ncon) = b.(char(endog(ii)))(end,r);        
    end
end

for r = 1:R
    for ii = 1:length(endog)
        gamma0BP(ii,r) = b.(char(endog(ii)))(2,r);
        gamma1BP(ii,r) = b.(char(endog(ii)))(3,r);
    end
end


for r = 1:R
    for i = 1:length(exog)
        AzwR{1,r}(i,:) = b.(exog{1,i})(2:end,r); 
    end
end

P0R = cell(1,R);    P1R = cell(1,R);      P2R = cell(1,R);

for r = 1:R
   P0R{1,r}(:,:) = [A0R{1,r}, -DwR{1,r}, -gamma0BP(:,r);
                     zeros(ncon+1,n), eye(ncon+1) ];
                 
   P1R{1,r}(:,:) = [A1R{1,r}, zeros(n, ncon), gamma1BP(:,r);
                     zeros(ncon, n), AzwR{1,r}, zeros(ncon, 1);
                     zeros(1,n), zeros(1, ncon), b.s(2,r)];
                 
   P2R{1,r}(:,:) = [A2R{1,r}, zeros(n, ncon+1);
                    zeros(ncon+1, ntot)];
   
    F1R{1,r} = P0R{1,r}\P1R{1,r};
    F2R{1,r} = P0R{1,r}\P2R{1,r};
    G_R{1,r} = eye(ntot);
    G_R{2,r} = F1R{1,r};
end



   
%% %% ############# IMPULSE RESPONSE FUNCTIONS ############# %% %%

for r = 1:R
    for h = 1:H
        G_R{h+2,r}(:,:) = F1R{1,r}*G_R{h+1,1} + F2R{1,r}*G_R{h,1} ;
    end
end

G_Rcumul = cell(H,R);
for r =1:R
    for h = 1:H
        if h ==1
            G_Rcumul{h,r}(:,:) = G_R{h,r};
        else
            G_Rcumul{h,r}(:,:) = G_R{h,r} + G_Rcumul{h-1,r} ;
        end
    end
end


%% Endogenous Policy shocks
for ind = 1:length(tot)
    for r = 1:R
        IR_BP.(tot{1,ind})(:,:,r) = zeros(ntot,H);
    end
end

% % All impulse responses for all shocks. IR grouped by the variables shocked
% where each column is the time horizon, each row is the variable reaction,
% each matrix is a replication. 
for ind = 1:length(tot)        
    for r = 1:R
        for h = 1:H
            IR_BP.(tot{1,ind})(:,h,r) = (G_R{h,r}*(inv(P0R{1,r}))*e{ind})*rmse.(tot{1,ind})(r,1);
        end
    end
end

% % Rows are the variables shocked, columns are the variables reacting
IRF1 = cell(ntot, ntot);
for k = 1:ntot
    for ind = 1:length(tot)
        for r = 1:R
            for h = 1:H
                IRF1{k,ind}(r,h) = IR_BP.(tot{1,k})(ind,h,r); 
            end
        end
    end
end

 for i = 1:ntot
     for k = 1:ntot
         IRF1{k,i}(:,:) = sort(IRF1{k,i}(:,:));
     end
 end
 
toc

hh = 0:H-1;
save('vars_for_IRFplots_altern_order.mat',  'hh', 'IRF1')    
clearvars -except alltab A* ay b C delta1 d IR

%The results can be plotted in the file "graphs_irfs.m"
% ############################# End ################################ %